=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2006-02-24 23:13:20 +0000
+++ bzrlib/reconcile.py	2006-02-25 00:32:59 +0000
@@ -30,8 +30,10 @@
 
     Currently this is limited to a inventory 'reweave'.
 
-    This is a convenience method, and the public api, for using a 
-    Reconciler object.
+    This is a convenience method, for using a Reconciler object.
+
+    Directly using Reconciler is recommended for library users that
+    desire fine grained control or analysis of the found issues.
     """
     reconciler = Reconciler(dir)
     reconciler.reconcile()
@@ -48,7 +50,14 @@
         self.bzrdir = dir
 
     def reconcile(self):
-        """Actually perform the reconciliation."""
+        """Perform reconciliation.
+        
+        After reconciliation the following attributes document found issues:
+        inconsistent_parents: The number of revisions in the repository whose
+                              ancestry was being reported incorrectly.
+        garbage_inventories: The number of inventory objects without revisions
+                             that were garbage collected.
+        """
         self.pb = ui.ui_factory.progress_bar()
         self.repo = self.bzrdir.open_repository()
         self.repo.lock_write()
@@ -64,14 +73,6 @@
         """Regenerate the inventory weave for the repository from scratch."""
         self.pb.update('Reading inventory data.')
         self.inventory = self.repo.get_inventory_weave()
-        self.repo.control_weaves.put_weave('inventory.backup',
-                                           self.inventory,
-                                           self.repo.get_transaction())
-        self.pb.note('Backup Inventory created.')
-        # asking for '' should never return a non-empty weave
-        new_inventory = self.repo.control_weaves.get_weave_or_empty('',
-            self.repo.get_transaction())
-
         # the total set of revisions to process
         self.pending = set([file_id for file_id in self.repo.revision_store])
 
@@ -81,10 +82,26 @@
 
         # mapping from revision_id to parents
         self._rev_graph = {}
+        # errors that we detect
+        self.inconsistent_parents = 0
         # we need the revision id of each revision and its available parents list
         for rev_id in self.pending:
             # put a revision into the graph.
             self._graph_revision(rev_id)
+        # we gc unreferenced inventories too
+        self.garbage_inventories = len(self.inventory.names()) \
+                                   - len(self._rev_graph)
+
+        if not self.inconsistent_parents and not self.garbage_inventories:
+            self.pb.note('Inventory ok.')
+            return
+        self.repo.control_weaves.put_weave('inventory.backup',
+                                           self.inventory,
+                                           self.repo.get_transaction())
+        self.pb.note('Backup Inventory created.')
+        # asking for '' should never return a non-empty weave
+        new_inventory = self.repo.control_weaves.get_weave_or_empty('',
+            self.repo.get_transaction())
 
         # we have topological order of revisions and non ghost parents ready.
         for rev_id in TopoSorter(self._rev_graph.items()).iter_topo_order():
@@ -121,6 +138,8 @@
             else:
                 mutter('found ghost %s', parent)
         self._rev_graph[rev_id] = parents   
+        if set(self.inventory.parent_names(rev_id)) != set(parents):
+            self.inconsistent_parents += 1
 
     def _reweave_step(self, message):
         """Mark a single step of regeneration complete."""

=== modified file 'bzrlib/tests/blackbox/test_reconcile.py'
--- bzrlib/tests/blackbox/test_reconcile.py	2006-02-24 23:13:20 +0000
+++ bzrlib/tests/blackbox/test_reconcile.py	2006-02-25 00:32:59 +0000
@@ -17,6 +17,7 @@
 """Black box tests for the reconcile command."""
 
 
+import bzrlib
 import bzrlib.bzrdir as bzrdir
 import bzrlib.repository as repository
 from bzrlib.tests import TestCaseWithTransport
@@ -41,6 +42,19 @@
         t = bzrdir.BzrDir.create_standalone_workingtree('.')
         (out, err) = self.run_bzr_captured(['reconcile'])
         self.assertEqualDiff(out, "Reconciling repository %s\n"
+                                  "Inventory ok.\n"
+                                  "Reconciliation complete.\n" %
+                                  t.bzrdir.root_transport.base)
+        self.assertEqualDiff(err, "")
+
+    def test_does_something_reconcile(self):
+        t = bzrdir.BzrDir.create_standalone_workingtree('.')
+        # an empty inventory with no revision will trigger reconciliation.
+        repo = t.branch.repository
+        inv = bzrlib.tree.EmptyTree().inventory
+        repo.add_inventory('missing', inv, [])
+        (out, err) = self.run_bzr_captured(['reconcile'])
+        self.assertEqualDiff(out, "Reconciling repository %s\n"
                                   "Backup Inventory created.\n"
                                   "Inventory regenerated.\n"
                                   "Reconciliation complete.\n" %

=== modified file 'bzrlib/tests/repository_implementations/test_reconcile.py'
--- bzrlib/tests/repository_implementations/test_reconcile.py	2006-02-23 04:08:56 +0000
+++ bzrlib/tests/repository_implementations/test_reconcile.py	2006-02-25 00:32:59 +0000
@@ -19,7 +19,7 @@
 
 import bzrlib
 import bzrlib.errors as errors
-from bzrlib.reconcile import reconcile
+from bzrlib.reconcile import reconcile, Reconciler
 from bzrlib.revision import Revision
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
 from bzrlib.transport import get_transport
@@ -27,10 +27,10 @@
 from bzrlib.workingtree import WorkingTree
 
 
-class TestNeedingReweave(TestCaseWithRepository):
+class TestsNeedingReweave(TestCaseWithRepository):
 
     def setUp(self):
-        super(TestNeedingReweave, self).setUp()
+        super(TestsNeedingReweave, self).setUp()
         
         t = get_transport(self.get_url())
         # an empty inventory with no revision for testing with.
@@ -40,8 +40,7 @@
 
         # a inventory with no parents and the revision has parents..
         # i.e. a ghost.
-        t.copy_tree('inventory_no_revision', 'inventory_one_ghost')
-        repo = bzrlib.repository.Repository.open('inventory_one_ghost')
+        repo = self.make_repository('inventory_one_ghost')
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -64,19 +63,33 @@
                        revision_id='the_ghost')
         rev.parent_ids = []
         repo.add_revision('the_ghost', rev)
-         
 
-    def test_reweave_empty_makes_backup_wave(self):
+    def test_reweave_empty(self):
         self.make_repository('empty')
         d = bzrlib.bzrdir.BzrDir.open('empty')
+        reconciler = Reconciler(d)
+        reconciler.reconcile()
+        # smoke test for reconcile appears to work too.
         reconcile(d)
+        # no inconsistent parents should have been found
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # and no backup weave should have been needed/made.
         repo = d.open_repository()
-        repo.control_weaves.get_weave('inventory.backup',
-                                      repo.get_transaction())
+        self.assertRaises(errors.NoSuchFile,
+                          repo.control_weaves.get_weave,
+                          'inventory.backup',
+                          repo.get_transaction())
 
     def test_reweave_inventory_without_revision(self):
         d = bzrlib.bzrdir.BzrDir.open('inventory_no_revision')
-        reconcile(d)
+        reconciler = Reconciler(d)
+        reconciler.reconcile()
+        # no inconsistent parents should have been found
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and one garbage inventories
+        self.assertEqual(1, reconciler.garbage_inventories)
         # now the backup should have it but not the current inventory
         repo = d.open_repository()
         backup = repo.control_weaves.get_weave('inventory.backup',
@@ -87,7 +100,13 @@
 
     def test_reweave_inventory_preserves_a_revision_with_ghosts(self):
         d = bzrlib.bzrdir.BzrDir.open('inventory_one_ghost')
-        reconcile(d)
+        reconciler = Reconciler(d)
+        reconciler.reconcile()
+        # no inconsistent parents should have been found: 
+        # the lack of a parent for ghost is normal
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and one garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
         # now the current inventory should still have 'ghost'
         repo = d.open_repository()
         repo.get_inventory('ghost')
@@ -97,7 +116,13 @@
         d = bzrlib.bzrdir.BzrDir.open('inventory_ghost_present')
         repo = d.open_repository()
         self.assertEqual([None, 'ghost'], repo.get_ancestry('ghost'))
-        reconcile(d)
+        reconciler = Reconciler(d)
+        reconciler.reconcile()
+        # one inconsistent parents should have been found : the
+        # available but not reference parent for ghost.
+        self.assertEqual(1, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
         # now the current inventory should still have 'ghost'
         repo = d.open_repository()
         repo.get_inventory('ghost')

